#include "ppc.h"
#include "m33.h"
#include <FL/Fl.H>
#include <FL/Fl_Gl_Window.H>
#include <GL/glut.h>


PPC::PPC(float hfov, int _w, int _h) : w(_w), h(_h) {

	C = V3(0.0f, 0.0f, 0.0f);
	a = V3(1.0f, 0.0f, 0.0f);
	b = V3(0.0f, -1.0f, 0.0f);
	float hfovd = hfov / 180.0f * 3.1415926f;
	c = V3(-(float)w / 2.0f, (float)h / 2.0f, -(float)w / (2.0f * tan(hfovd / 2.0f)));

}


int PPC::Project(V3 P, V3 &pP) {

	M33 M;
	M.SetColumn(0, a);
	M.SetColumn(1, b);
	M.SetColumn(2, c);
	V3 q = M.Inverted()*(P - C);
	if (q[2] <= 0.0f)
		return 0;

	pP[0] = q[0] / q[2];
	pP[1] = q[1] / q[2];
	pP[2] = 1.0f / q[2];
	return 1;

}

V3 PPC::GetVD() {

	return (a^b).UnitVector();

}

void PPC::Rotate(V3 aDir, float theta) {

	a = a.RotateThisVector(aDir, theta);
	b = b.RotateThisVector(aDir, theta);
	c = c.RotateThisVector(aDir, theta);

}

float PPC::GetF() {

	return c*GetVD();

}

void PPC::PositionAndOrient(V3 Cn, V3 L, V3 upg) {

	V3 vdn = (L - Cn).UnitVector();
	V3 an = (vdn ^ upg).UnitVector();
	V3 bn = vdn ^ an;
	float f = GetF();
	V3 cn = vdn*f - an* (float)w / 2.0f - bn* (float)h / 2.0f;

	a = an;
	b = bn;
	c = cn;
	C = Cn;

}

void PPC::Unproject(V3 pP, V3 &P) {

	P = C + (a*pP[0] + b*pP[1] + c) / pP[2];

}


void PPC::SetIntrinsicsHW(float nearz, float farz) {

	glViewport(0, 0, w, h);
	float scf = nearz / GetF();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-(float)w / 2.0f*scf, (float)w / 2.0f*scf, -(float)h / 2.0f*scf, (float)h / 2.0f*scf,
		nearz, farz);
	glMatrixMode(GL_MODELVIEW);

}


void PPC::SetExtrinsicsHW() {

	glLoadIdentity();
	V3 L = C + GetVD()*100.0f;
	gluLookAt(C[0], C[1], C[2], L[0], L[1], L[2], -b[0], -b[1], -b[2]);

}

M33 PPC::GetMatrix() {

	M33 ret;
	ret.SetColumn(0, a);
	ret.SetColumn(1, b);
	ret.SetColumn(2, c);
	return ret;

}

void PPC::Zoom(float flscf) {

	// multiplies focal length by flscf
	float f = GetF();
	float nf = f * flscf;
	V3 nc = c + GetVD()*(nf - f);
	c = nc;

}